@@ -26,6 +26,7 @@
 #ifndef __MLVIEW_APP_CONTEXT_H__
 #define __MLVIEW_APP_CONTEXT_H__
 
+#include "config.h"
 #include <libxml/parser.h>
 #include <libxml/catalog.h>
 #include <glib.h>
@@ -76,6 +77,27 @@ struct _MlViewAppContextClass {
          *class of MlViewAppContext.
          */
         GObjectClass parent_class;
+
+        /**
+         *Signal emited when the MlViewApp has been initialyzed.
+         */
+        void (*application_initialized) (MlViewAppContext *a_this,
+                                        gpointer a_user_data) ;
+
+        /**
+         *signal emited when an editing
+         *widget has been requested to popup
+         *a contextual menu.
+         */
+        void (*contextual_menu_requested) (MlViewAppContext *a_this,
+                                           GtkWidget *a_request_source,
+                                           GdkEvent *a_event,
+                                           gpointer a_user_data) ;
+
+        void (*view_changed) (MlViewAppContext *a_this,
+                              gpointer a_old_view,
+                              gpointer a_new_view,
+                              gpointer a_user_data) ;
 };
 
 
@@ -100,6 +122,8 @@ enum MlViewTreeEditorsNodeColour {
 struct MlViewAppSettings {
         struct {
                 gboolean validation_is_on ;
+                gboolean enable_completion_box;
+                gchar *default_editing_view_type ;
         } general ;
         struct {
                 gint default_doc_expansion_depth ;
@@ -183,9 +207,22 @@ gint mlview_app_context_get_xpm (MlViewAppContext * a_this,
                                  GdkBitmap ** a_bitmap);
 
 void mlview_app_context_set_xml_catalog (MlViewAppContext * a_this,
-                                         xmlCatalog * a_xml_catalog);
+                                         xmlCatalog * a_xml_catalog) ;
 
 xmlCatalog *mlview_app_context_get_xml_catalog (MlViewAppContext* a_this);
 
+
+enum MlViewStatus mlview_app_context_notify_contextual_menu_request (MlViewAppContext *a_this,
+                                                                     GtkWidget *a_source_widget,
+                                                                     GdkEvent *a_event) ;
+
+enum MlViewStatus mlview_app_context_notify_view_swapped (MlViewAppContext *a_this,
+                                                          gpointer a_old_view,
+                                                          gpointer a_new_view) ;
+
+enum MlViewStatus mlview_app_context_notify_application_initialized (MlViewAppContext *a_this) ;
+
+gboolean mlview_app_context_ask_internal_subset_node_name (gchar **a_name) ;
+
 G_END_DECLS
 #endif /*__MLVIEW-APP-CONTEXT_H__*/
