@@ -1,4 +1,4 @@
-/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 8-*- */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  *This file is part of MlView.
  *
@@ -57,22 +57,6 @@ static const gint MAX_NUMBER_OF_ELEMENT_NAMES_IN_CHOICE_LIST = 256;
 
 static GtkWidget *gv_dtd_choice_dialog = NULL;
 
-/*
-#ifdef MLVIEW_DATA_DIR
-static const gchar 
-* gv_mlview_data_dir = MLVIEW_DATA_DIR ;
-#else
-static const gchar * gv_mlview_data_dir = NULL ;
-#endif
-
-#ifdef MLVIEW_XML_CATALOG_DIR_NAME
-static const gchar * gv_mlview_xml_catalog_dir_name = 
-MLVIEW_XML_CATALOG_DIR_NAME ;
-#else
-static const gchar * gv_mlview_xml_catalog_dir_name = NULL ;
-#endif
-*/
-
 typedef struct _MlViewParserCtxt MlViewParserCtxt;
 
 /**
@@ -115,7 +99,9 @@ static xmlDtdPtr custom_xmlSAXParseDTD (MlViewAppContext * a_app_context,
 
 
 static gboolean mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app_context,
-                                                                        const gchar * a_external_id, const gchar * a_system_id,
+                                                                        const gchar * a_external_id, 
+                                                                        const gchar * a_system_id,
+                                                                        const xmlDoc *a_doc,
                                                                         MlViewExtSubsDef ** a_ext_subs_def);
 
 static xmlChar *mlview_resolve_external_entity (MlViewAppContext * a_app_ctxt,
@@ -182,7 +168,6 @@ mlview_ext_subs_def_clone (MlViewExtSubsDef * a_ext_subs_def)
 
 /**
  *destroys the MlViewExtSubsDef given in argument. 
- *
  */
 void
 mlview_ext_subs_def_destroy (MlViewExtSubsDef * a_def) 
@@ -301,6 +286,9 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
                 /*overload the externalSubset sax handler */
                 xml_ctxt->sax->externalSubset =
                         mlview_external_subset_sax_handler;
+
+                xml_ctxt->sax->resolveEntity =
+                        mlview_sax_resolve_entity;
                 /*
                  *tell the custom sax handler to save the 
                  *external subset definition ... or not
@@ -336,8 +324,7 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
         /*restore the error display stream of libxml */
         xmlSetGenericErrorFunc (NULL, NULL);
         if (a_app_context
-            &&
-            !mlview_app_context_error_buffer_is_empty
+            && !mlview_app_context_error_buffer_is_empty
             (a_app_context)) {
                 mlview_app_context_display_buffered_error
                         (a_app_context);
@@ -357,7 +344,6 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
         return (parse_status == XML_ERR_OK) ? 0 : parse_status;
 }
 
-
 /**
  *A custom entity loader that first fetches 
  *the entities by resolving their url reference
@@ -470,6 +456,12 @@ mlview_resolve_external_entity (MlViewAppContext * a_app_context,
                               (a_app_context), FALSE);
 
         result = xmlCatalogResolve (a_pub_id, a_sys_id);
+        if (!result && a_sys_id) {
+                if (g_file_test (a_sys_id, G_FILE_TEST_EXISTS) == TRUE
+                    && g_file_test (a_sys_id, G_FILE_TEST_IS_DIR) == FALSE) {
+                        result = g_strdup (a_sys_id) ;
+                }
+        }
         return result;
 }
 
@@ -494,7 +486,7 @@ mlview_external_subset_sax_handler (void *a_ctxt,
 {
         xmlParserCtxt *ctxt = NULL;
         struct MlViewAppSettings *settings = NULL ;
-
+        gchar *system_id = NULL ;
         g_return_if_fail (a_ctxt);
 
         ctxt = a_ctxt;
@@ -508,12 +500,13 @@ mlview_external_subset_sax_handler (void *a_ctxt,
                 g_return_if_fail (settings) ;
         }
 
-        if (settings->validation_is_on == TRUE
+        if (settings->general.validation_is_on == TRUE
             && ctxt) {
                 ctxt->validate = 1;
         } else {
                 ctxt->validate = 0;
         }
+        system_id = (gchar*)a_system_id ;
 
         /*
          *If validation is on, ask the user if she wants 
@@ -527,7 +520,7 @@ mlview_external_subset_sax_handler (void *a_ctxt,
                 gboolean validate =
                         mlview_parsing_utils_ask_for_DTD_change_and_validation
                         (gv_app_ctxt, a_external_id,
-                         a_system_id, &ext_subs_def);
+                         system_id, ctxt->myDoc, &ext_subs_def);
 
                 if (validate == FALSE)
                         ctxt->validate = 0;
@@ -535,32 +528,35 @@ mlview_external_subset_sax_handler (void *a_ctxt,
                 if (validate == TRUE && ext_subs_def) {
                         a_external_id =
                                 ext_subs_def->external_id;
-
-                        a_system_id =
+                        system_id =
                                 ext_subs_def->system_id;
                 }
         }
 
         if (ctxt->validate == 1) {
+#ifdef LIBXML_2_6_SERIES_OR_ABOVE
+                xmlSAX2ExternalSubset (ctxt,
+                                       a_name, a_external_id,
+                                       system_id) ;
+#else
                 externalSubset (ctxt,
                                 a_name, a_external_id,
-                                a_system_id);
-
+                                system_id);
+#endif /*LIBXML_2_6_SERIES_OR_ABOVE*/
                 if (ctxt->myDoc
                     && (ctxt->myDoc->extSubset == NULL)) {
                         mlview_app_context_warning
                                 (gv_app_ctxt,
-                                 _
-                                 ("The external DTD subset was not found. I couldn't validate the document."));
+                                 _("The external DTD subset was not found. I couldn't validate the document."));
                 }
-        }
 
-        /*
-         *We will not do validation during the document parsing.
-         *We will only do post parsing validation 
-         *(if validation is required).
-         */
-        ctxt->validate = 0;
+                /*
+                 *We will not do validation during the document parsing.
+                 *We will only do post parsing validation 
+                 *(if validation is required).
+                 */
+                ctxt->validate = 0;
+        }
 }
 
 
@@ -670,7 +666,7 @@ custom_xmlSAXParseDTD (MlViewAppContext * a_app_context,
 
         xmlFreeParserCtxt (ctxt);
 
-        return (ret);
+                return (ret);
 }
 
 /**
@@ -693,32 +689,51 @@ static gboolean
 mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app_context,
                                                         const gchar * a_external_id,
                                                         const gchar * a_system_id,
+                                                        const xmlDoc *a_doc,
                                                         MlViewExtSubsDef ** a_ext_subs_def)
 {
         GladeXML *gxml = NULL;
         gchar *gfile = NULL ;
         gint button = 0;
-        gboolean result = TRUE;
+        gboolean result = TRUE, is_relative = FALSE;
         GtkWidget *label_public_id = NULL;
         GtkWidget *label_system_id = NULL;
         GtkWidget *label_resource = NULL;
-        const gchar *public_id = NULL;
-        const gchar *system_id = NULL;
+        gchar *public_id = NULL;
+        gchar *system_id = NULL;
         xmlChar *resource = NULL;
 
         g_return_val_if_fail (a_app_context != NULL, TRUE);
         g_return_val_if_fail (a_ext_subs_def != NULL,
                               TRUE);
 
-        public_id = ((const gchar *) (a_external_id != NULL)) ?
-                a_external_id : "\"\"";
-        system_id = ((const gchar *) (a_system_id != NULL)) ?
-                a_system_id : "\"\"";
+        if (a_external_id != NULL)
+                public_id = (gchar*)a_external_id ;
+        else 
+                public_id = (gchar*)"\"\"";
+
+        if (a_system_id != NULL) 
+                system_id = (gchar*)a_system_id ;
+        else system_id = (gchar*)"\"\"";
+
         *a_ext_subs_def = NULL; /*sanity initialization */
+        system_id = g_strdup (system_id) ;
+
+        mlview_utils_uri_is_relative (system_id, &is_relative) ;
+        if (is_relative == TRUE) {
+                gchar *tmp_str = NULL ;
+                mlview_utils_relative_uri_to_absolute_uri (system_id,
+                                                           a_doc->URL,
+                                                           &tmp_str) ;
+                if (tmp_str) {
+                        system_id = tmp_str ;
+                        tmp_str = NULL ;
+                }
+        }
         resource =
                 mlview_resolve_external_entity (a_app_context,
                                                 a_external_id,
-                                                a_system_id);
+                                                system_id);
         if (resource) {
                 gfile = gnome_program_locate_file
                         (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
@@ -773,11 +788,7 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
                          "mlview_dtd_choice_dtd_not_resolved") ;
         }
         glade_xml_signal_autoconnect (gxml);
-        mlview_app_context_set_window_icon 
-                (a_app_context, GTK_WINDOW (gv_dtd_choice_dialog));
-        mlview_app_context_set_window_icon (a_app_context,
-                                            GTK_WINDOW
-                                            (gv_dtd_choice_dialog));
+
         button = gtk_dialog_run (GTK_DIALOG
                                  (gv_dtd_choice_dialog));
         if (resource) {
@@ -857,7 +868,9 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
         gtk_widget_hide (GTK_WIDGET (gv_dtd_choice_dialog));
         g_object_unref (gxml);
         g_free (gfile);
-
+        if (system_id) {
+                g_free (system_id) ;
+        }
         return result;
 }
 
@@ -1121,8 +1134,7 @@ g_list_compare_string_elems (gchar * a_str1, gchar * a_str2)
  */
 xmlDoc *
 mlview_parsing_utils_load_xml_file (gchar * a_file_name,
-                                    MlViewAppContext *
-                                    a_app_context)
+                                    MlViewAppContext *a_app_context)
 {
         xmlDoc *result = NULL;
         xmlParserCtxtPtr parser_context = NULL;
@@ -1164,7 +1176,6 @@ mlview_parsing_utils_load_xml_file (gchar * a_file_name,
         return result;
 }
 
-
 /**
  *Saves an xml file using the application wide settings. 
  *@param a_xml_doc the xml document to save.
@@ -1211,11 +1222,12 @@ mlview_parsing_utils_load_a_dtd (MlViewExtSubsDef * a_subset_def,
                 g_assert (sax_handler != NULL);
 
                 memset (sax_handler, 0, sizeof (xmlSAXHandler));
-
+#ifdef LIBXML_2_6_SERIES_OR_ABOVE
+                xmlSAX2InitDefaultSAXHandler(sax_handler, 0) ;
+#else
                 initxmlDefaultSAXHandler (sax_handler, 0);
-
+#endif
                 g_assert (sax_handler != NULL);
-
                 /*override the libxml error printing routine */
                 xmlSetGenericErrorFunc
                         (a_app_context, (xmlGenericErrorFunc)
@@ -1275,7 +1287,7 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
 {
         gint result = 0;
         gint validity_status;
-        xmlValidCtxt validation_context;
+        xmlValidCtxt validation_context = {0};
 
         g_return_val_if_fail (a_doc != NULL, -1);
 
@@ -1285,9 +1297,11 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
         validation_context.warning = (xmlValidityWarningFunc)
                 mlview_app_context_bufferize_error;
 
+
         xmlSetGenericErrorFunc (a_app_context,
                                 (xmlGenericErrorFunc)
                                 mlview_app_context_bufferize_error);
+        
         mlview_app_context_set_error_dialog_title
                 (a_app_context,
                  _
@@ -1297,10 +1311,6 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
                 xmlValidateDtd (&validation_context, a_doc,
                                 a_dtd);
 
-        xmlSetGenericErrorFunc
-                (a_app_context, (xmlGenericErrorFunc)
-                 mlview_app_context_bufferize_error);
-
         if (validity_status == 1) {
                 /*xmlValidate() returns 1 if doc is valid 
                  *whereas we want to return 0 in that case ...*/
@@ -1317,6 +1327,9 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
                 mlview_app_context_set_error_dialog_title
                         (a_app_context, NULL);
 
+        /*restore the generic error handling routine*/
+        xmlSetGenericErrorFunc (NULL, NULL) ;
+
         return result;
 }
 
@@ -1372,18 +1385,16 @@ mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
                 (a_app_context, a_title);
         g_return_val_if_fail (filesel != NULL, NULL);
 
-        gtk_widget_realize (GTK_WIDGET (filesel));
-        mlview_app_context_set_window_icon (a_app_context,
-                                            GTK_WINDOW
-                                            (filesel));
-        button_clicked =
-                mlview_file_selection_run (filesel, TRUE);
+        button_clicked = mlview_file_selection_run (filesel, TRUE);
 
         switch (button_clicked) {
         case OK_BUTTON:
                 dtd = g_strdup (gtk_file_selection_get_filename
                                 (GTK_FILE_SELECTION (filesel)));
-                if (dtd) {
+                if (dtd && strcmp (dtd, "") 
+                    && g_file_test (dtd, G_FILE_TEST_IS_DIR))
+                        break ;
+                if (dtd && strcmp (dtd, "")) {
                         subset_def =
                                 mlview_ext_subs_def_new (NULL, NULL, 
                                                          dtd);
@@ -1426,6 +1437,7 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
         gint nb_of_names = 0;
         struct MlViewAppSettings *settings = NULL ;
 
+        xmlGenericError = NULL ;
         g_return_val_if_fail (a_current_xml_node != NULL, -2);
         g_return_val_if_fail (a_current_xml_node->type
                               == XML_ELEMENT_NODE, -2);
@@ -1441,7 +1453,7 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
                 g_return_val_if_fail (settings, -2) ;
         }
 
-        if (settings->validation_is_on != TRUE)
+        if (settings->general.validation_is_on != TRUE)
                 return -1; /*validation is not turned on !!! */
 
         if ((a_insertion_scheme == INSERT_BEFORE)
@@ -1556,6 +1568,7 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
                                      g_list_compare_string_elems);
         }
 
+        xmlSetGenericErrorFunc (NULL, NULL) ;
         return nb_of_names;
 }
 
@@ -1593,7 +1606,7 @@ mlview_parsing_utils_build_attribute_name_completion_list (MlViewAppContext * a_
                 g_return_val_if_fail (settings, -2) ;
         }
 
-        if (settings->validation_is_on != TRUE )
+        if (settings->general.validation_is_on != TRUE )
                 return -1;     /*validation is not turned on !!! */
 
         if (a_current_xml_node->doc->intSubset != NULL)
@@ -1674,7 +1687,7 @@ mlview_parsing_utils_get_element_content_table (MlViewAppContext * a_app_context
                 (a_app_context) ;
         g_return_val_if_fail (settings, BAD_PARAMETER) ;
                 
-        if (settings->validation_is_on != TRUE) {
+        if (settings->general.validation_is_on != TRUE) {
                 return -1;/*validation is not turned on !!! */
         }
         if (!*a_element_content_table) {
@@ -1757,7 +1770,7 @@ mlview_parsing_utils_build_required_attributes_list (MlViewAppContext * a_app_co
         settings = mlview_app_context_get_settings
                 (a_app_context) ;
         g_return_val_if_fail (settings, BAD_PARAMETER) ;
-        if (settings->validation_is_on != TRUE) {
+        if (settings->general.validation_is_on != TRUE) {
                 return VALIDATION_IS_OFF;
         }
 
@@ -1902,7 +1915,7 @@ mlview_parsing_utils_build_required_children_tree (MlViewAppContext * a_app_cont
         settings = mlview_app_context_get_settings 
                 (a_app_context) ;
         g_return_val_if_fail (settings, BAD_PARAMETER) ;
-        if (settings->validation_is_on != TRUE) {
+        if (settings->general.validation_is_on != TRUE) {
                 return VALIDATION_IS_OFF;
         }
 
@@ -2137,3 +2150,4 @@ mlview_parsing_utils_build_graphical_attr_values (MlViewAppContext * a_app_conte
 
         return result;
 }
+
