@@ -70,16 +70,7 @@ typedef struct _MlViewTreeEditor2Private MlViewTreeEditor2Private;
  *that can be caught and handler by the caller. 
  */
 struct _MlViewTreeEditor2 {
-        GtkVBox vbox;
-        
-        /*
-         *a cache that contains
-         *a list of xmlNode/GtkTreeRowReference
-         *pair. This is to speed up searches.
-         */
-        GHashTable *nodes_rows_hash;
-        MlViewAppContext *app_context;
-
+        GtkVBox vbox;               
         MlViewTreeEditor2Private *priv;
 };
 
@@ -95,12 +86,43 @@ struct _MlViewTreeEditor2 {
 
 guint mlview_tree_editor2_get_type (void);
 
-GtkWidget *mlview_tree_editor2_new (MlViewAppContext *
-                                    a_app_context);
+GtkWidget *mlview_tree_editor2_new (MlViewAppContext *a_app_context);
 
 void mlview_tree_editor2_construct (MlViewTreeEditor2 *editor, 
                                     MlViewAppContext *a_context);
 
+enum MlViewStatus mlview_tree_editor2_internal_general_entity_to_string (MlViewTreeEditor2 *a_this,
+                                                                         xmlEntity *a_entity,
+                                                                         guchar **a_string) ;
+
+enum MlViewStatus mlview_tree_editor2_external_general_parsed_entity_to_string (MlViewTreeEditor2 *a_this,
+                                                                                xmlEntity *a_entity,
+                                                                                guchar **a_string) ;
+
+enum MlViewStatus mlview_tree_editor2_external_general_unparsed_entity_to_string (MlViewTreeEditor2 *a_this,
+                                                                                  xmlEntity *a_entity,
+                                                                                  guchar **a_string) ;
+
+enum MlViewStatus mlview_tree_editor2_internal_parameter_entity_to_string (MlViewTreeEditor2 *a_this,
+                                                                           xmlEntity *a_entity,
+                                                                           guchar **a_string) ;
+
+enum MlViewStatus mlview_tree_editor2_external_parameter_entity_to_string (MlViewTreeEditor2 *a_this,
+                                                                           xmlEntity *a_entity,
+                                                                           guchar **a_string) ;
+
+enum MlViewStatus mlview_tree_editor2_dtd_node_to_string (MlViewTreeEditor2 *a_this,
+                                                          xmlDtd *a_dtd_node,
+                                                          guchar **a_string) ;
+
+enum MlViewStatus mlview_tree_editor2_entity_ref_to_string (MlViewTreeEditor2 *a_this,
+                                                            xmlNode *a_node,
+                                                            guchar **a_string) ;
+
+enum MlViewStatus mlview_tree_editor2_cdata_section_to_string (MlViewTreeEditor2 *a_this, 
+                                                               xmlNode *a_node, 
+                                                               guchar **a_result) ;
+
 GtkTreeModel *mlview_tree_editor2_get_model (MlViewTreeEditor2 *a_this);
 
 void mlview_tree_editor2_destroy (GtkObject * a_object);
@@ -216,6 +238,9 @@ enum MlViewStatus mlview_tree_editor2_update_child_node_added (MlViewTreeEditor2
                                                                xmlNode * a_node, 
                                                                gboolean a_emit_signals);
 
+enum MlViewStatus mlview_tree_editor2_update_internal_subset_added (MlViewTreeEditor2 *a_this,
+                                                                    xmlDtd *a_subset_node) ;
+
 enum MlViewStatus mlview_tree_editor2_update_node_cut  (MlViewTreeEditor2 * a_this,
                                                         xmlNode * a_parent_node,
                                                         xmlNode * a_node_cut);
@@ -245,12 +270,31 @@ enum MlViewStatus mlview_tree_editor2_connect_to_doc (MlViewTreeEditor2 *a_this,
 enum MlViewStatus mlview_tree_editor2_disconnect_from_doc (MlViewTreeEditor2 *a_this,
                                                            MlViewXMLDocument *a_doc) ;
 
-guchar *mlview_tree_editor2_build_attrs_list_str (xmlNode *a_node);
+enum MlViewStatus mlview_tree_editor2_edit_xml_entity_decl_node (MlViewTreeEditor2 *a_this,
+                                                                 xmlEntity *a_entity_node,
+                                                                 guchar *a_new_text) ;
 
-const gchar *mlview_tree_editor2_get_colour_string (xmlElementType a_type);
+enum MlViewStatus mlview_tree_editor2_edit_dtd_node (MlViewTreeEditor2 *a_this,
+                                                     xmlDtd *a_dtd_node,
+                                                     guchar *a_new_text) ;
 
+enum MlViewStatus mlview_tree_editor2_edit_cdata_section_node (MlViewTreeEditor2 *a_this,
+                                                               xmlNode *a_cur_node,
+                                                               gchar *a_new_text) ;
+
+guchar * mlview_tree_editor2_build_attrs_list_str (MlViewTreeEditor2 *a_this,
+                                                   xmlNode * a_node) ;
+
+const gchar *mlview_tree_editor2_get_colour_string (MlViewTreeEditor2 *a_this,
+                                                    xmlElementType a_type);
+
+GHashTable * mlview_tree_editor2_get_nodes_rows_hash (MlViewTreeEditor2 *a_this) ;
+
+void mlview_tree_editor2_set_nodes_rows_hash (MlViewTreeEditor2 *a_this,
+                                              GHashTable *a_nodes_rows_hash) ;
 enum MlViewTreeInsertType {
-        INSERT_TYPE_ADD_CHILD,
+        INSERT_TYPE_APPEND_CHILD_VISIT_NEXT,
+        INSERT_TYPE_PREPEND_CHILD,
         INSERT_TYPE_INSERT_BEFORE,
         INSERT_TYPE_INSERT_AFTER
 };
