@@ -135,7 +135,7 @@ struct _MlViewXMLDocumentClass {
                                         gpointer a_user_data) ;
 
         void (*node_attribute_added) (MlViewXMLDocument *a_xml_doc,
-                                      xmlAttr *a_attr, gpointer *a_user_data) ;
+                                      xmlAttr *a_attr, gpointer a_user_data) ;
 
         void (*node_namespace_added) (MlViewXMLDocument *a_this,
                                       xmlNode *a_node,
@@ -161,11 +161,44 @@ struct _MlViewXMLDocumentClass {
 
         void (*searched_node_found) (MlViewXMLDocument *a_xml_doc,
                                      xmlNode *a_node_found,
-                                     gpointer *a_user_data) ;
+                                     gpointer a_user_data) ;
 
         void (*node_selected) (MlViewXMLDocument *a_xml_doc,
                                xmlNode *a_node_found,
-                               gpointer *a_user_data) ;
+                               gpointer a_user_data) ;
+
+        void (*node_unselected) (MlViewXMLDocument *a_xml_doc,
+                                 xmlNode *a_node,
+                                 gpointer a_user_data) ;
+
+        void (*dtd_node_system_id_changed) (MlViewXMLDocument *a_this,
+                                            xmlDtd *a_dtd_node,
+                                            gpointer a_user_data) ;
+
+        void (*dtd_node_public_id_changed) (MlViewXMLDocument *a_this,
+                                            xmlDtd *a_dtd_node,
+                                            gpointer a_user_data) ;
+
+        void (*dtd_node_created) (MlViewXMLDocument *a_this,
+                                  xmlDtd *a_dtd_node,
+                                  gpointer a_user_data) ;
+
+        void (*entity_node_content_changed) (MlViewXMLDocument *a_this,
+                                            xmlEntity *a_entity_node,
+                                            gpointer a_user_data) ;
+
+        void (*entity_node_public_id_changed) (MlViewXMLDocument *a_this,
+                                               xmlEntity *a_entity_node,
+                                               gpointer a_user_data) ;
+
+        void (*entity_node_system_id_changed) (MlViewXMLDocument *a_this,
+                                              xmlEntity *a_entity_node,
+                                              gpointer *a_user_data) ;
+
+        void (*dtd_associated) (MlViewXMLDocument *a_this,
+                                MlViewExtSubsDef *a_ext_subs_def,
+                                gpointer a_user_data);
+
 };
 
 guint mlview_xml_document_get_type (void);
@@ -175,15 +208,17 @@ MlViewXMLDocument *mlview_xml_document_new (xmlDocPtr
                                             MlViewAppContext
                                             * a_context);
 
-void
-mlview_xml_document_ref (MlViewXMLDocument * a_this);
+void mlview_xml_document_ref (MlViewXMLDocument * a_this);
 
-void
-mlview_xml_document_unref (MlViewXMLDocument * a_this);
+void mlview_xml_document_unref (MlViewXMLDocument * a_this);
 
 MlViewXMLDocument *mlview_xml_document_open (gchar * a_file_name,
                                              MlViewAppContext *a_context);
 
+MlViewXMLDocument *mlview_xml_document_open_with_dtd (gchar * a_file_name,
+                                                      gchar * a_dtd_name,
+                                                      MlViewAppContext * a_context);
+
 xmlNode *mlview_xml_document_get_node_from_clipboard (xmlDoc * a_xml_doc);
 MlViewXMLDocument * mlview_xml_document_clone (MlViewXMLDocument *a_original);
 
@@ -278,6 +313,12 @@ xmlNode *mlview_xml_document_set_node_name (MlViewXMLDocument * a_xml_document,
                                             enum MlViewEncoding a_encoding,
                                             gboolean a_emit_signal);
 
+enum MlViewStatus mlview_xml_document_set_entity_node_name (MlViewXMLDocument *a_this,
+                                                            xmlEntity *a_entity,
+                                                            xmlDtd *a_dtd_node,
+                                                            guchar *a_name,
+                                                            gboolean a_emit_signal) ;
+
 enum MlViewStatus mlview_xml_document_node_get_name (xmlNode * a_node,
                                                      enum MlViewEncoding a_enc, 
                                                      guchar ** a_outbuf);
@@ -307,6 +348,12 @@ enum MlViewStatus  mlview_xml_document_synch_attributes (MlViewXMLDocument *a_do
                                                          xmlNode *a_node,
                                                          GList *a_nv_pair_list) ;
 
+enum MlViewStatus mlview_xml_document_create_internal_subset (MlViewXMLDocument *a_this,
+                                                              xmlChar * a_name,
+                                                              xmlChar *public_id,
+                                                              xmlChar *system_id,
+                                                              gboolean a_emit_signal) ;
+
 xmlNs * mlview_xml_document_create_ns (MlViewXMLDocument *a_this,
                                        xmlNode *a_node, guchar *uri,
                                        guchar *prefix,
@@ -324,6 +371,31 @@ enum MlViewStatus mlview_xml_document_remove_ns (MlViewXMLDocument *a_this,
                                                  xmlNode *a_node,
                                                  gboolean a_emit_signal) ;
 
+enum MlViewStatus mlview_xml_document_set_dtd_node_system_id (MlViewXMLDocument *a_this,
+                                                              xmlDtd *a_dtd,
+                                                              xmlChar *a_system_id,
+                                                              gboolean a_emit_signal) ;
+
+enum MlViewStatus mlview_xml_document_set_dtd_node_public_id (MlViewXMLDocument *a_this,
+                                                              xmlDtd *a_dtd,
+                                                              xmlChar *a_public_id,
+                                                              gboolean a_emit_signal) ;
+
+enum MlViewStatus mlview_xml_document_set_entity_content (MlViewXMLDocument *a_this,
+                                                          xmlEntity *a_entity,
+                                                          xmlChar *a_content,
+                                                          gboolean a_emit_signal) ;
+
+enum MlViewStatus mlview_xml_document_set_entity_public_id (MlViewXMLDocument *a_this,
+                                                            xmlEntity *a_entity,
+                                                            xmlChar *a_external_id,
+                                                            gboolean a_emit_signal) ;
+
+enum MlViewStatus mlview_xml_document_set_entity_system_id (MlViewXMLDocument *a_this,
+                                                            xmlEntity *a_entity,
+                                                            xmlChar *a_system_id,
+                                                            gboolean a_emit_signal) ;
+
 enum MlViewStatus mlview_xml_document_node_get_fqn_len_as_isolat1 (xmlNode * a_node, gint * a_len);
 
 enum MlViewStatus mlview_xml_document_node_get_fqn (xmlNode * a_node, 
@@ -348,5 +420,11 @@ enum MlViewStatus mlview_xml_document_search3 (MlViewXMLDocument *a_this,
                                                xmlNode *a_from,
                                                xmlNode **a_found,
                                                gboolean a_emit_signal);
+
+gboolean mlview_xml_document_is_completion_possible_global (MlViewXMLDocument *a_this);
+
+gboolean mlview_xml_document_is_node_valid (MlViewXMLDocument *a_this,
+                                            xmlNode *a_node);
+
 G_END_DECLS
 #endif
